/**
 * App component script
 */
import { AfterViewInit, Component, ViewChild, ViewEncapsulation } from '@angular/core';
import { AssignmentModel, CalendarIntervalModel, CalendarModel, Gantt, ProjectModel, ResourceModel, TaskModel } from '@bryntum/gantt';
import {
    BryntumResourceUtilizationComponent
} from '@bryntum/gantt-angular';

import {
    projectModelConfig,
    resourceUtilizationConfig
} from './app.config';
import assignments from '../assets/data/launch-saas.json';

@Component({
    selector: 'app-root',
    templateUrl: './app.component.html',
    styleUrls: ['./app.component.scss'],
    encapsulation: ViewEncapsulation.None
})
export class AppComponent {
    public projectModelConfig = projectModelConfig;
    public resourceUtilizationConfig = resourceUtilizationConfig;
    public projectModel: ProjectModel = new ProjectModel(this.projectModelConfig);

    @ViewChild('resourceUtilization')
    resourceUtilizationComponent!: BryntumResourceUtilizationComponent;
    constructor() {
        this.projectModel.onLoad = () => {
            this.projectModel.taskStore.data = assignments.map((a) => new TaskModel({
                id: a.ResourceAllocationId.toString(),
                name: a.Name,
                manuallyScheduled: true,
                startDate: new Date(a.StartDateTime),
                endDate: new Date(a.EndDateTime),
            })).sort((a, b) => a.name.localeCompare(b.name));

            this.projectModel.assignmentStore.data = assignments.map((a) =>
                new AssignmentModel({
                    id: a.ResourceAllocationId.toString(),
                    eventId: a.ResourceAllocationId.toString(),
                    resourceId: a.ContactId.toString(),
                    units: this.calculateUnits(a),
                }));
        }
    }

    private calculateUnits(assignment: any): number | undefined {
        const start = new Date(assignment.StartDateTime.toString());
        const end = new Date(assignment.EndDateTime.toString());
        if (assignment.WorkAmount === 0) {
            return 0;
        }
        try {
            const resource = this.projectModel.resourceStore.getById(assignment.ContactId.toString()) as ResourceModel;
            const calendar = this.projectModel.calendarManagerStore.getById(resource?.calendar) as CalendarModel;
            if (calendar !== undefined) {
                const duration = calendar.calculateDurationMs(start, end);
                if (duration === 0) {
                    //duration = this.calculateNonWorkingTimeUnits(assignment);
                    if (duration === 0) {
                        return 0;
                    }
                }
                return assignment.WorkAmount / (duration / 1000 / 60 / 60) * 100;
            }
            return undefined;
        } catch {
            return undefined;
        }
    }

    private calculateNonWorkingTimeUnits(assignment: any): number {
        if (assignment.StartDateTime >= assignment.EndDateTime) {
            return 0;
        }
        const task = this.projectModel.taskStore.getById(assignment.ResourceAllocationId.toString()) as TaskModel;
        let taskCalendar = this.projectModel.calendars.find((c) => c.id === task.id) as CalendarModel;
        if (taskCalendar === null) {
            taskCalendar = new CalendarModel({
                id: task.id,
                name: task.name,
                intervals: [
                    new CalendarIntervalModel({
                        isWorking: true,
                        startDate: assignment.StartDateTime,
                        endDate: assignment.EndDateTime,
                    })
                ]
            });
            this.projectModel.calendars.push(taskCalendar);
        }
        task.ignoreResourceCalendar = true;
        task.calendar = taskCalendar;

        return taskCalendar.calculateDurationMs(
            new Date(assignment.StartDateTime.toString()),
            new Date(assignment.EndDateTime.toString()));
    }
}
